import numpy as np
from skcomponents.processing.processor import SignalProcessor


class electons_per_bin(SignalProcessor):
    """
     Compute the number of electrons generated by the incoming signal. This assumes the incoming signal has already been
     converted to photon counts per second at the detector plane

     Signal_out =  (signal_in[photons/s])*quantum_efficiency*time[s] + dark_current[electrons/s]*time[s] + Offsets
                                                                                + Readout_electrons

    """

    def __init__(self, specs, integration_time, add_dark=True):

        self.qe = specs.DetectorQE
        self.specs = specs
        self.time = integration_time
        self.add_dark = add_dark
        self.dark_current = specs.DetectorDark
        self.read_noise = specs.DetectorReadNoise
        self.bias = specs.DetectorBias

    def process_signal(self, signal: np.ndarray):

        if self.add_dark:

            dark = self.dark_current * self.time * np.ones(signal.shape)

            return self.qe * signal * self.time + dark + self.bias

        if not self.add_dark:

            return self.qe * signal * self.time
